use strict;
use warnings;

package SmbGate;

use Filesys::SmbClient 3.1;
use Carp;
our $VERSION = '1.0';

=head1 NAME

SmbGate - Handles login, file and directory interaction with the SMB server

=head1 SYNOPSIS

    $smb = SmbGate->new($maxfilesize);

Upon instantiation, a SmbGate object is not worth much - It waits for
other code to signal it to start its interaction.

If $maxfilesize is given (it should be a positive intever), it will
determine how large (in bytes) it will allow the maximum file transfer
sizes to be. If it is not a positive , no file limit will be enforced.

=cut

sub new {
    my ($class, $self, $maxsize);
    $class = shift;
    $maxsize = shift;
    $maxsize = undef unless $maxsize =~ /^\d+$/;
    $self = {maxsize => $maxsize};
    return bless $self, $class;
}

=pod

    $ok = $smb->login($share, $login, $passwd);

Tries to log in to the specified share, using the user login and
password passed as parameters. 

C<$share> must be an URL for a SMB resource (i.e.
C<smb://server/share>). C<$login> and C<$passwd> should be a valid
username/password combination to access the given share.

Note that SmbClient will only work for user-based (not share-based)
resources.

If the login is successful, it returns with a true value, and sets the
internal information to remember this share/login/password
combination. If the login fails, it returns undef.

=cut

sub login {
    my ($self);
    $self = shift;

    $self->{share} = shift;
    $self->{login} = shift;
    $self->{passwd} = shift;

    $self->{auth} = undef;

    unless ($self->{share} =~ m!smb://[\w\d_]+/[\w\d_]+$!) {
	carp 'Share incorrectly specified - Should be in the ' .
	  "smb://server/share form (is: $self->{share})";
    }

    $self->{smb} = Filesys::SmbClient->new(username => $self->{login},
					   password => $self->{passwd},
					   debug => 0
					   );
    $self->{smb}->set_flag(SMBCCTX_FLAG_NO_AUTO_ANONYMOUS_LOGON);

    # How can we be sure the login succeeded? By opening the share and reading
    # its root directory
    unless ($self->_get_dir_handle('/')) {
	carp "Error opening share $self->{share} as user $self->{login}";
	return undef;
    }

    $self->{auth} = 1;
    return 1;
}

=pod

    $ok = $smb->is_logged_in;

Returns true if there is a currently authenticated user state in the
object, false otherwise.

=cut

sub is_logged_in {
    my $self = shift;
    return $self->{auth};
}

=pod

    $files = $smb->dir($directory);

Returns the list of files (and their corresponding types) in the given
directory in the currently logged in share. If no directory is
specified, defaults to C</>.

The returned structure is a hashref, where each element is a hashref
with this structure:

For directories:

    { name => $filename, type => 'dir' }

For regular files:

    { name => $filename, type => 'file', size => $filesize,
      mode => $access_mode, mtime => $mtime }

Besides the obvious, $access_mode holds the access permissions for
this file, and $mtime holds the last modification time for the file
(refer to L<stat> for further details).

SmbGate handles only files and directories. For any other tree objects
found in the requested directory, we will only report:

    { name => $filename, type => 'other' }

The returned file listing is sorted alphabetically, case-insensitively.

=cut

sub dir {
    my ($self, $dir, $dir_handle, @files);
    $self = shift;
    $dir = shift || '/';

    unless ($self->is_logged_in) {
	carp "Not currently logged in to a SMB server";
	return undef;
    }

    $dir_handle = $self->_get_dir_handle($dir) or return undef;

    while (my $entry = $self->{smb}->readdir_struct($dir_handle)) {
	my ($type, $filename) = @$entry;
	next if ($filename eq '.' or $filename eq '..');

	# Type can be any of the following constants (see Filesys::SmbClient): 
	# SMBC_WORKGROUP, SMBC_SERVER, SMBC_FILE_SHARE, SMBC_PRINTER_SHARE,
	# SMBC_COMMS_SHARE, SMBC_IPC_SHARE, SMBC_DIR, SMBC_FILE, SMBC_LINK
	# We will just deal with plain files and directories.
	if ($type == SMBC_DIR) {
	    push @files, {name => $filename, type => 'dir'};

	} elsif ($type == SMBC_FILE) {
	    my @stat = $self->{smb}->stat("$self->{share}/$dir/$filename");
	    push @files, {name => $filename, type => 'file', size => $stat[7],
			  mode => $stat[2], mtime => $stat[11]};

	} else {
	    push @files, {name => $filename, type => 'other'};
	}
    }
    $self->{smb}->close($dir_handle);

    # Return the directory sorted alphabetically, non-case-sensitively
    return [ sort {lc($a->{name}) cmp lc($b->{name})} @files ];
}

=pod

    $data = $smb->read_file($filespec);

Reads a full file ($filespec should include the full directory/file
specification inside the currently logged-in share), and returns it to
the caller. 

Note that the full file is read B<to RAM>, and abuse of this method
can lead to a DoS. This module strives to provide the right
functionality - Remember to specify a maximum download size!

=cut

sub read_file {
    my ($self, $file, $fd, @ret);
    $self = shift;
    $file = shift;

    unless ($self->is_logged_in) {
	carp "Not currently logged in to a SMB server";
	return undef;
    }

    if ($self->{maxsize}) {
	my @stat = $self->{smb}->stat("$self->{share}/$file");
	if ($stat[7] > $self->{maxsize}) {
	    warn "$file is too long ($stat[7] bytes, maximum is " .
		$self->{maxsize};
	    return undef;
	}
    }

    unless ($fd = $self->{smb}->open("$self->{share}/$file",0400)) {
	warn "Could not open $self->{share}/$file: $!";
	return undef;
    }

    while (my $buf = $self->{smb}->read($fd)) {
	push @ret, $buf
    }
    $self->{smb}->close($fd);

    return join '', @ret;
}

######################################################################
# Private methods
sub _get_dir_handle {
    my ($self, $dir, $dh, $uri);
    $self = shift;
    $dir = shift || '/';

    $uri = "$self->{share}/$dir";

    unless ($self->{smb}) {
	carp "Not connected to SMB server";
	return undef;
    }

    unless ($dh = $self->{smb}->opendir($uri)) {
	carp "Error opening $dir";
	return undef;
    }

    return $dh;
}

1;

=head1 REQUIRES

L<Filesys::SmbClient> version 3.1 or later

=head1 SEE ALSO

=head1 AUTHOR

Gunnar Wolf, gwolf@gwolf.org
Instituto de Investigaciones Econmicas, UNAM

=head1 COPYRIGHT

Copyright 2006-2007 Gunnar Wolf

This library is free software, you can redistribute it and/or modify it
under the terms of the GPL version 2 or later.

=cut

1;
