use strict;
use warnings;

package SmbGate::Embperl::Baseconf;
use Carp;
use YAML;
our $AUTOLOAD;
our $VERSION = $SmbGate::VERSION;

=head1 NAME

SmbGate::Embperl::BaseConf - Handles the system-wide configuration for SmbGate

=head1 SYNOPSIS

    $conf = SmbGate::Embperl::BaseConf->new($instance, $filename);

$filename is the full path to the YAML configuration for SmbGate
(defaults to C</etc/smbgate.yaml> if not specified). $instance is the
SmbGate instance for which this configuration object will
act (defaults to C<smbgate>).

If the requested instance does not exist in the configuration file, an
exception will be raised.

The instance refers to a hash-like heading where the configuration
keys will be. Take the following L<YAML> file as a working example:

    ---
    smbgate:
      base_url: /smb/
      static_dir: /usr/local/lib/SmbGate/embperl/static/
      smbhost: myserver
      allow_debug: 0
    testserver:
      base_url: /test/
      static_dir: /usr/local/lib/SmbTest/embperl/static/
      smbhost: otherserver
      allow_debug: 1

This configuration file can be accessed for two different SmbGate
instances: C<smbgate> and C<testserver>. The example C<$conf> object
will refer only to the first.

Please refer to the L<YAML> documentation for further information on
its format.

=cut

sub new {
    my ($class, $self, $instance, $conffile);
    $class = shift;
    $instance = shift || 'smbgate';
    $conffile = shift || '/etc/smbgate.yaml';
    $self = {};
    bless $self, $class;

    $self->_get_defaults;
    if (-e $conffile and -r $conffile) {
	$self->_parse_conf($instance, $conffile);
    } else {
	carp "$conffile not found or not readable - falling back to defaults.";
    }

    return $self;
}

sub DESTROY {
    # Just return... Keep AUTOLOAD happy
    1;
}

=pod

=head2 Accessing the configuration data

C<SmbGate::Embperl::Baseconf> does not know (or care, for that matter)
about the entries it manages - It just hands back whatever key is
requested from it:

    $res = $conf->base_url;

    $res = $conf->static_dir;

    $res = $conf->smbhost;

    $res = $conf->allow_debug;

If a requested key is not found, an exception is raised.

=cut

######################################################################
# Accessors: Handled via AUTOLOAD

sub AUTOLOAD {
    my ($self, $name, $myclass);
    $self = shift;
    $name = $AUTOLOAD;

    # AUTOLOAD gives us the fully qualified name of the requested method - 
    # Strip the classname and the last double colon off it.
    $myclass = ref($self);
    if (index($name, $myclass)) {
	carp "Invocation error - Class $myclass, method requested $AUTOLOAD";
	return undef;
    } 
    substr($name, 0, length($myclass)+2, '');

    if (exists $self->{attr}{$name}) {
	return $self->{attr}{$name};
    }
    croak "Invalid configuration key '$name' requested";
}

=pod

If you need the list of currently defined keys, you can fetch it by calling:

    @keys = $conf->defined_keys;

=cut

sub defined_keys {
    my ($self);
    $self = shift;
    return sort keys %$self;
}

######################################################################
# Private methods, for internal use only

sub _get_defaults {
    my $self = shift;

    $self->{attr}{session_timeout} = 10; # Default session timeout: 10 minutes

    return 1;
}

sub _parse_conf {
    my ($self, $instance, $conffile, %defaults, $data);
    $self = shift;
    $instance = shift;
    $conffile = shift;

    # YAML takes care of error reporting - Be happy :)
    $data = YAML::LoadFile($conffile);

    defined $data->{$instance} or
	croak "Instance $instance not defined in configuration file $conffile";

    for my $key (keys %{$data->{$instance}}) {
	# The key is explicitly specified? If not, keep the default
	$self->{attr}{$key} = $data->{$instance}{$key};
    }

    return 1;
}

=head1 REQUIRES

L<Carp>, L<YAML>

=head1 SEE ALSO

L<SmbGate>, L<SmbGate::Embperl>

=head1 AUTHOR

Gunnar Wolf, gwolf@gwolf.org
Instituto de Investigaciones Econmicas, UNAM

=head1 COPYRIGHT

For copying information, please refer to L<SmbGate>, included with
this module, or directly contact the author.

=cut

1;
