use strict;
use warnings;

package SmbGate::Embperl::HtmlHelper;
use HTML::Entities qw();
our $VERSION = $SmbGate::VERSION;

=head1 NAME

SmbGate::Embperl::HtmlHelper - HTML functions used for SmbGate's
Embperl component

=head1 SYNOPSIS

This is just a collection of loosely relateed HTML-producing functions.

use SmbGate::Embperl::HtmlHelper;

    $html = SmbGate::Embperl::HtmlHelper->new(base => $base, user => $user);

Where $base is the (absolute or relative) URL for the system's root,
and $user is a string with the current user's login.

=cut

sub new {
    my ($class, %init, $self);
    $class = shift;
    %init = @_;
    $self = {base => $init{base}, user => $init{user}};
    return bless $self, $class;
}

=pod

    $size = $html->human_size($amount);

Returns a human-readable string representing the received amount in
bytes, KB, MB or GB.

=cut

sub human_size {
    my ($self, $amount, $res, @max);
    $self = shift;
    $amount = shift;

    return "" unless defined $amount;

    @max = ([2**30, 'GB'],
	    [2**20, 'MB'],
	    [2**10, 'KB'],
	    [1, 'Bytes']);

    for my $scale (@max) {
	next if $amount/1024 > $scale->[0];
	$res = $amount / $scale->[0];
	$res =~ s/^(\d+\.\d).*/$1/;
	$res .= " $scale->[1]";
    }

    return $res;
}

=pod

    $link = $html->link_to($dir, $file);

Creates an internal link to the requested file. $file is a
directory/file entry (as generated by L<SmbGate>), $dir is a regular
string with the directory where $file is located.

=cut

sub link_to {
    my ($self, $dir, $file, $res);
    $self = shift;
    $dir = shift;
    $file = shift;

    return $file->{name} unless ($file->{type} =~ /^(?:dir|file)$/);

    $res = '<a href="' . $self->{base};
    if ($file->{type} eq 'dir') {
	$res .= "dir/$dir/$file->{name}";
    } else {
	$res .= "file/$dir/$file->{name}";
    }
    $res .= "\">$file->{name}</a>";

    # Sometimes I just get too many slashes together. They don't change the
    # meaning, but look quite ugly! Convert them all to single slashes (unless
    # preceded by a colon, as it might be an URL)
    $res =~ s/(?<!:)\/+/\//g;

    return $res;
}

=pod

    @dirs = $html->parent_dirs($dir)

Returns a list of arrays representing -starting at the system's root-
the full path of directories under which the specified directory is
nested. $dir is just the directory as a string. Each returned array
has as the first element the partial directory (starting at the user's
root) for the requested element, and as the second element just the
last element (i.e. for sending as a label). You can refer for its use
to L<embperl/index.html>.

=cut

sub parent_dirs {
    my ($self, $parent, $path, @parents);
    $self = shift;
    $parent = shift || '';
    $parent =~ s!/+!/!g; 

    $path = '';
    @parents = (['/', '\\\\iiec\\' . $self->{user}]);

    for my $part_dir (split m!/!, $parent) {
	next unless $part_dir;
	$path .= "$part_dir/";
	push @parents, [$path, $part_dir];
    }

    return @parents;
}

=pod

    $link = $html->static_file($file);

Returns a HTML link to the URL for the specified file's full
contents. $file is just the full filespec - directory and file name

=cut

sub static_file {
    my ($self, $file);
    $self = shift;
    $file = shift;
    return "$self->{base}/static?file=$file";
}

=pod

=head1 REQUIRES

L<HTML::Entities>

=head1 SEE ALSO

L<SmbGate>, L<SmbGate::Embperl>

=head1 AUTHOR

Gunnar Wolf, gwolf@gwolf.org
Instituto de Investigaciones Econmicas, UNAM

=head1 COPYRIGHT

For copying information, please refer to L<SmbGate>, included with
this module, or directly contact the author.

=cut

1;
