use strict;
use warnings;

package SmbGate::Embperl::Warnings;
our $VERSION = $SmbGate::VERSION;

=head1 NAME

SmbGate::Embperl::Warnings - Stores and retrieves warnings and errors
for SmbGate

=head1 SYNOPSIS

SmbGate::Embperl::Warnings provides a two-level, FIFO-like structure
for storing and retrieving messages at different stages of your
program's flow.

    $warn = SmbGate::Embperl::Warnings->new($errors, $warnings);

$errors and $warnings should be array references. If they are empty to
begin with, they can be left out, but the object can be instantiated
with some messages already defined.

=cut

sub new {
    my ($class, $msg, $err, $self);
    $class = shift;
    $err = shift || [];
    $msg = shift || [];

    $self = {err => $err, msg => $msg};
    unless (ref($self->{err}) eq 'ARRAY' and ref($self->{msg}) eq 'ARRAY') {
	die sprintf("Both arguments should be array references (%s, %s)",
		    ref $self->{err}, ref $self->{msg});
    }

    return bless $self, $class;
}

=pod

    $warn->add_msg($message);

    $warn->add_err($error);

Adds a message or an error to the end of its respective queue.

=cut

sub add_msg { $_[0]->_add_item('msg', $_[1]); }
sub add_err { $_[0]->_add_item('err', $_[1]); }

=pod

    $message = $warn->get_next_msg;

    $error = $warn->get_next_err;

Retrieves the registered messages/errors in the order they were
registered (FIFO).

=cut

sub get_next_msg { $_[0]->_get_item('msg'); }
sub get_next_err { $_[0]->_get_item('err'); }

sub _add_item {
    my ($self, $level, $msg);
    $self = shift;
    $level = shift;
    $msg = shift;

    # Avoid storing duplicated messages
    push @{$self->{$level}}, $msg unless 
	grep {$_ eq $msg} @{$self->{$level}};;
    return 1;
}

sub _get_item {
    my ($self, $level);
    $self = shift;
    $level = shift;

    return undef unless scalar @{$self->{$level}};
    return shift @{$self->{$level}};
}

=head1 SEE ALSO

L<SmbGate>, L<SmbGate::Embperl>

=head1 AUTHOR

Gunnar Wolf, gwolf@gwolf.org
Instituto de Investigaciones Econmicas, UNAM

=head1 COPYRIGHT

For copying information, please refer to L<SmbGate>, included with
this module, or directly contact the author.

=cut

1;
