use strict;
use warnings;

=head1 NAME

SmbGate::Embperl::Webclient - Non-content-related interaction with the
Web client

=head1 SYNOPSIS

This module handles the interaction with the Web client that is not
part of the generated HTML pages - this is, headers handling, debug
status setting, etc.

    $webclient = SmbGate::Embperl::Webclient->new($epreq, $headers, 
                                                  $udat, $allow_debug);

C<$epreq> is a reference to the Embperl request object (precisely,
what Embperl makes available as C<$epreq> to its C<base.pm> modules).

$headers is a reference to the C<%http_headers_out> hash given (again)
to its modules.

$udat is (you guessed right!) a reference to the Embperl C<%udat>
hash.

$allow_debug is just a boolean value, indicating if we should allow
debugging information to be displayed back to the browser (see below
at the C<set_debug> method for further details).

=cut

package SmbGate::Embperl::WebClient;
our $VERSION = $SmbGate::VERSION;

sub new {
    my ($class, $self);
    $class = shift;
    $self = {epreq => shift, headers => shift, udat => shift, 
	     allow_debug => shift};
    return bless $self, $class;
}

=pod

    $webclient->redirect($destination)

Redirects the request to a new page inside the system (preceding it by
the configured base URL) - and stops processing before taking any
further actions.

=cut

sub redirect {
    my ($self, $to);
    $self = shift;
    $to = $self->{epreq}->{conf}->base_url . '/' . shift;

    $to =~ s!/+!/!g;

    $self->{headers}{Location} = $to;
    Embperl::exit(301);
}

=pod

    $webclient->content_type($type);

Sets the generated HTTP content type to whatever value is specified.

=cut

sub content_type {
    my ($self, $type);
    $self = shift;
    $type = shift;

    $self->set_header('Content-Type', $type);
}

=pod

    $webclient->set_header($header, $value);

Sets the specified header to the given value. If no value is specified
(or an C<undef> value is received, the requested header will be unset.

=cut

sub set_header {
    my ($self, $header, $data);
    $self = shift;
    $header = shift;
    $data = shift;

    if (defined $data) { 
	$self->{headers}{$header} = $data;
    } else {
	delete $self->{headers}{$header};
    }
}

=pod

    $webclient->user_agent

Returns the user agent string that started this request. 

=cut

sub user_agent {
    return $ENV{HTTP_USER_AGENT};
}

=pod

    $webclient->set_debug;

    $webclient->unset_debug;

Sets or unsets the system's debugging mode - When in debug mode, the
generated HTML includes a section with debugging information.

B<Important>: Currently, no authorization is carried out to allow
specific users to set/unset the debug flag. Debugging information
B<includes sensitive information> such as the user's password. Make
sure your production systems are configured not to allow debugging information to be displayed!

=cut

sub set_debug {
    my $self = shift;
    return unless $self->{allow_debug};
    $self->{udat}{debug} = 1;
}

sub unset_debug {
    my $self = shift;
    # If debug is turned on, just let anybody switch it off.
    $self->{udat}{debug} = 0;
}

=head1 SEE ALSO

L<SmbGate>, L<SmbGate::Embperl>

=head1 AUTHOR

Gunnar Wolf, gwolf@gwolf.org
Instituto de Investigaciones Econmicas, UNAM

=head1 COPYRIGHT

For copying information, please refer to L<SmbGate>, included with
this module, or directly contact the author.

=cut

1;
